using System.Collections.Generic;
using System.Linq;
using System;
using Microsoft.Xna.Framework;

namespace CoronaVirusGame.GuiGame
{
    public class OpponentAi : Person
    {
        private Vector2 _lastTarget;
        private bool _movingForward;
        public override void Update(GameTime gameTime, GuiGameDisplay game)
        {
            var nonEmptyShelves = game.Shelves.Where(s => s.ToiletPaperCount != 0 && s.Accessable);
            Shelf closestShelf = Shelf.GetClosestShelf(nonEmptyShelves, Posistion);
            if (ToiletPaperCount != ToiletPaperCapacity && closestShelf != null)
            {
                var interesctingShelves = nonEmptyShelves.Where(s => s.SelectionRange.Intersects(Rectangle) && s.Accessable);
                if (interesctingShelves.Count() > 0)
                {
                    _movingForward = false;
                    if (TpPickupCooldown == null || _canPickup)
                    {
                        AddToiletPaper();
                        Shelf targetShelf = Shelf.GetClosestShelf(interesctingShelves, Posistion);
                        targetShelf.ToiletPaperCount--;
                        _lastTpPickup = DateTime.Now;
                    }
                }
                else
                {
                    Vector2 target = closestShelf.DrawRectangle.Center.ToVector2();
                    if (target != _lastTarget)
                        _movingForward = false;
                    MoveToTarget(gameTime, target);
                    _lastTarget = target;
                }
            }
            else if (ToiletPaperCount == 0 && closestShelf == null)
            {
                _movingForward = false;
                return;
            }
            else
            { //TODO: In the future, maybe select the closest checkout
                Checkout checkout = game.Checkouts[0];
                MoveToTarget(gameTime, checkout.DrawRectangle.Center.ToVector2());
                if (checkout.IsSelectable(Rectangle, out _))
                    _movingForward = false;
            }
        }
        private void MoveToTarget(GameTime gameTime, Vector2 target)
        {
            float direction = (float)Math.Atan2(target.Y - OriginInGame.Y, target.X - OriginInGame.X);
            float pi2 = (float)(Math.PI * 2);
            direction = (direction + pi2) % pi2;
            if (_movingForward || Math.Round((decimal)Rotation - (decimal)(Math.Truncate((Rotation / pi2)) * pi2), 1) == Math.Round((decimal)direction, 1))
            {
                MoveForward(MovementSpeed, gameTime);
                _movingForward = true;
            }
            else
            {
                float difference = direction - Rotation;
                if (difference > Math.PI)
                    difference = ((float)(2 * Math.PI) - difference);
                if (difference < -Math.PI)
                    difference = ((float)(2 * Math.PI) + difference);
                float rotationPerFrame = 2.5f * (float)(gameTime.ElapsedGameTime.TotalSeconds);
                float rotationNeeded = (difference < 0) ? -rotationPerFrame : rotationPerFrame;
                Rotation += rotationNeeded;
            }
        }
    }
}