using System;
using System.Collections.Generic;
using CoronaVirusGame.GuiGame;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using MonoGame.Extended;

namespace CoronaVirusGame.GuiGame
{
    public abstract class Person
    {
        public static string GetIdentifier(ref int opponentIndex, Type personType)
        {
            string identifier;
            if (personType == typeof(Player))
                identifier = "Player";
            else if (personType == typeof(OpponentAi))
            {
                identifier = $"Opponent #{opponentIndex}";
                opponentIndex++;
            }
            else
                identifier = "Unidentifiable";
            return identifier;
        }
        public Texture2D Texture;
        public Vector2 Posistion;
        public Vector2 Origin
        {
            get
            {
                return new Vector2(Texture.Bounds.Size.X / 2, Texture.Bounds.Size.Y / 2);
            }
        }
        public Vector2 OriginInGame
        {
            get
            {
                return (Origin * new Vector2((PersonSize.X / Texture.Bounds.Width))) + Posistion; 
            }
        }
        public Rectangle Rectangle
        {
            get
            {
                return new Rectangle(
                    (int)(Posistion.X - (PersonSize.X / 2)),
                    (int)(Posistion.Y - (PersonSize.Y / 2)),
                    (int)PersonSize.X, (int)PersonSize.Y
                );
            }
        }
        public float Rotation;
        public float MovementSpeed;
        public Texture2D TpTexture;
        public uint ToiletPaperCount;
        public uint ToiletPaperCapacity = 20; //TODO: Tweak this number
        public Point TpSize;
        public Point2 PersonSize;
        public int Score;
        protected DateTime _lastTpPickup;
        protected bool _canPickup
        {
            get
            {
                return (DateTime.Now - _lastTpPickup) > TpPickupCooldown;
            }
        }
        public int PotentialAdditionalScore
        {
            get
            {
                return (int)ToiletPaperCount * 5;
            }
        }
        public TimeSpan TpPickupCooldown;
        private List<Vector2> TpPositions = new List<Vector2>(); //TODO: Should these be vectors?
        public void AddToiletPaper()
        {
            if (ToiletPaperCount == ToiletPaperCapacity)
                throw new ArgumentException("Person is over capacity");
            Random rnd = new Random();
            TpPositions.Add(
                new Vector2(
                    rnd.Next(-(int)(PersonSize.X / 2), (int)(PersonSize.X / 2)),
                    rnd.Next(-(int)(PersonSize.Y / 2), (int)(PersonSize.Y / 2))
                )
            );
            ToiletPaperCount++;
        }
        public void ResetToiletPaper()
        {
            ToiletPaperCount = 0;
            TpPositions = new List<Vector2>();
        }
        public void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(Texture,
                Posistion,
                null,
                Color.White,
                Rotation,
                Origin,
                PersonSize.X / Texture.Bounds.Width, //This makes the assumption that the size's aspect ratio is the same as the texture's
                SpriteEffects.None,
                0
            );
            foreach (Vector2 tpPos in TpPositions)
                spriteBatch.Draw(TpTexture, Vector2.Transform(tpPos, Matrix.CreateRotationZ(Rotation)) + Posistion, null, Color.White, Rotation, new Vector2(TpSize.X / 2, TpSize.Y / 2), (float)TpSize.Y / (float)TpTexture.Height, SpriteEffects.None, 1);
        }
        public abstract void Update(GameTime gameTime, GuiGameDisplay game);
        protected void MoveForward(float speed, GameTime gameTime)
        {
            Vector2 direction = new Vector2((float)Math.Cos(Rotation), (float)Math.Sin(Rotation));
            direction.Normalize();
            Posistion += direction * speed * (float)gameTime.ElapsedGameTime.TotalSeconds;
        }
    }
}