using System.Linq;
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using MonoGame.Extended;
using System.Collections.Generic;

namespace CoronaVirusGame.GuiGame
{
    public class Restocker : Person
    {
        private RestockerManager _manager;
        public bool OutsideGame
        {
            get
            {
                return Posistion.X + PersonSize.X < 0;
            }
        }
        private Shelf _lastShelf;
        private Shelf _currentTarget;
        private int _restocksLeft;
        public Restocker(RestockerModel model, RestockerManager manager)
        {
            base.PersonSize = model.Size;
            base.Texture = model.Texture;
            base.TpPickupCooldown = model.TpPickupCooldown;
            base.ToiletPaperCapacity = model.ToiletPaperCapacity;
            this._manager = manager;
            Posistion = new Vector2(-30, -30);
        }
        public override void Update(GameTime gameTime, GuiGameDisplay game)
        {
            if (_currentTarget == null)
                _currentTarget = _manager.RequestShelf(Posistion);
            float MovementSpeed = 500 * (float)gameTime.ElapsedGameTime.TotalSeconds;
            if ( (_currentTarget != null && ToiletPaperCount > 0) && !(OutsideGame && ToiletPaperCount != ToiletPaperCapacity) )
            {
                if (_currentTarget != _lastShelf)
                {
                    if (_lastShelf != null)
                        _lastShelf.Accessable = true;
                    _lastShelf = _currentTarget;
                    _restocksLeft = (int)(_currentTarget.ToiletPaperCapacity - _currentTarget.ToiletPaperCount);
                }
                float xDistance = _currentTarget.DrawRectangle.X - Posistion.X;
                float yDistance = _currentTarget.DrawRectangle.Y - Posistion.Y;
                if (_currentTarget.DrawRectangle.X != Posistion.X)
                    Posistion.X += AdvanceToTarget(xDistance, GetCurrentMovementSpeed(MovementSpeed, xDistance));
                else if (_currentTarget.DrawRectangle.Y != Posistion.Y)
                    Posistion.Y += AdvanceToTarget(yDistance, GetCurrentMovementSpeed(MovementSpeed, yDistance));
                else if (_canPickup)
                {
                    ToiletPaperCount--;
                    _currentTarget.ToiletPaperCount++;
                    _currentTarget.Accessable = false;
                    _lastTpPickup = DateTime.Now;
                    _restocksLeft--;
                    if (_restocksLeft == 0 || _currentTarget.ToiletPaperCapacity == _currentTarget.ToiletPaperCount)
                        _currentTarget = null;
                }
            }
            else if (!OutsideGame)
            {
                if (_lastShelf != null)
                {
                    _lastShelf.Accessable = true;
                    _lastShelf = null;
                }
                Posistion.X -= MovementSpeed;
            }
            else if (OutsideGame)
            {
                if (ToiletPaperCount != ToiletPaperCapacity)
                    ToiletPaperCount++;
            }
        }
        private static float AdvanceToTarget(float targetDistance, float movementSpeed)
        {
            float targetPosAbs = Math.Abs(targetDistance);
            float movementSpeedAbs = Math.Abs(movementSpeed);
            return movementSpeedAbs < targetPosAbs ? movementSpeed : targetDistance;
        }
        private static float GetCurrentMovementSpeed(float movementSpeed, float distance)
        {
            return distance >= 0 ? movementSpeed : -movementSpeed;
        }
    }
}