using System.Linq;
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace CoronaVirusGame.GuiGame
{
    public class RestockerManager
    {
        private GuiGameDisplay _game;
        public List<Restocker> Restockers = new List<Restocker>();
        private List<Shelf> _shelfPool;
        private DateTime? _lastCycleFinish;
        public TimeSpan TimeInbetweenCycles;
        public Shelf RequestShelf(Vector2 pos, bool removeFromPool = true)
        {
            if (_shelfPool.Count == 0)
                return null;
            Shelf returnValue = Shelf.GetClosestShelf(_shelfPool, pos);
            if (removeFromPool)
                _shelfPool.Remove(returnValue);
            return returnValue;
        }
        public RestockerManager(GuiGameDisplay game, RestockerModel model, int restockers, TimeSpan timeInbetweenCycles = default(TimeSpan))
        {
            _game = game;
            TimeInbetweenCycles = timeInbetweenCycles;
            for (int i = 0; i < restockers; i++)
            {
                Restocker toAdd = new Restocker(model, this);
                Restockers.Add(toAdd);
            }
            NextCycle();
        }
        public void NextCycle()
        {
            var nonFullShelves = _game.Shelves.Where(s => s.ToiletPaperCount != s.ToiletPaperCapacity);
            _shelfPool = new List<Shelf>(nonFullShelves);
        }
        public bool ShouldCycleProgess()
        {
            if (_shelfPool.Count == 0 && Restockers.All(r => r.OutsideGame))
            {
                if (_lastCycleFinish == null)
                    _lastCycleFinish = DateTime.Now;
                if ( (DateTime.Now - _lastCycleFinish) > TimeInbetweenCycles)
                {
                    _lastCycleFinish = null;
                    return true;
                }
            }
            return false;
        }
        public void Update(GameTime gameTime)
        {
            foreach (Restocker restocker in Restockers)
                restocker.Update(gameTime, _game);
            if (ShouldCycleProgess())
                NextCycle();
        }
        public void DrawAllRestockers(SpriteBatch spriteBatch)
        {
            foreach (Restocker restocker in Restockers)
            {
                restocker.Draw(spriteBatch);
            }
        }
    }
}