using System.Linq;
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Graphics;
using MonoGame.Extended;
using Microsoft.Xna.Framework;

namespace CoronaVirusGame.GuiGame
{
    public class Scoreboard
    {
        public static string[] GetScoreStrings(List<Person> persons)
        {
            List<Person> orderedPersons = persons.OrderByDescending(p => p.Score).ToList();
            List<string> returnValue = new List<string>();
            Person winner = orderedPersons.First();
            int opponentIndex = 1;
            int highestScore = 0;
            string winnerString = string.Empty;
            var scoreStrings = new List<(int score, string scoreString)>();
            foreach (Person person in persons)
            {
                string identifier = Person.GetIdentifier(ref opponentIndex, person.GetType());
                string toAdd = $"{identifier} scored {person.Score} pounds";
                if (highestScore < person.Score)
                {
                    highestScore = person.Score;
                    winnerString = $"{identifier} wins the game!";
                } 
                scoreStrings.Add((person.Score, toAdd));
            }
            returnValue.Add(winnerString);
            returnValue.AddRange(scoreStrings.OrderByDescending(s => s.score).Select(S => S.scoreString));
            return returnValue.ToArray();
        }
        public static Rectangle GetDrawRectangle(string[] scoreStrings, SpriteFont font, Point resolution)
        {
            Point pos = new Point(resolution.X / 2, resolution.Y / 2);
            Vector2 measuredString = font.MeasureString("S");
            int maxLength = scoreStrings.Max(s => (int)font.MeasureString(s).X);
            int height = (int)measuredString.Y; //This casting may need to change
            height *= scoreStrings.Length;
            pos = new Point(pos.X - (maxLength / 2), pos.Y - (height / 2));
            return new Rectangle(pos, new Size(maxLength, height));
        }   
        private SpriteFont _font;
        public Rectangle DrawRectangle;
        private string[] ScoreStrings; 
        public Scoreboard(List<Person> persons, SpriteFont font, Point resolution)
        {
            ScoreStrings = GetScoreStrings(persons);
            DrawRectangle = GetDrawRectangle(ScoreStrings, font, resolution);
            _font = font;
        }
        public void Draw(SpriteBatch spriteBatch)
        { //TODO: Change colouring
            spriteBatch.FillRectangle(DrawRectangle, Color.White);
            Vector2 stringPos = DrawRectangle.Location.ToVector2();
            float incrementY = _font.MeasureString("S").Y;
            foreach (string score in ScoreStrings)
            {
                spriteBatch.DrawString(_font, score, stringPos, Color.Black);
                stringPos.Y += incrementY;
            }
        }
    }
}