using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using MonoGame.Extended;

namespace CoronaVirusGame.GuiGame
{
    public class Shelf : Selectable
    {
        public Rectangle DrawRectangle;
        public uint ToiletPaperCount;
        public uint ToiletPaperCapacity;
        public Texture2D ShelfTexture;
        public Texture2D TpTexture;
        public Point TpSize;
        private List<Vector2> TpPosistions;
        public bool Accessable = true;
        public static Shelf GetClosestShelf(IEnumerable<Shelf> Shelves, Vector2 posistion)
        {
            var distances = new Dictionary<float, Shelf>();
            foreach (Shelf shelf in Shelves)
            {
                Vector2 shelfPos = shelf.DrawRectangle.Location.ToVector2();
                float distance = Vector2.Distance(shelfPos, posistion);
                try
                {
                    distances.Add(distance, shelf);
                }
                catch (ArgumentException)
                {
                    //Ignore; don't add
                }
            }
            if (distances.Count == 0)
                return null;
            float lowestDistance = distances.Min(d => d.Key);
            Shelf returnValue = distances[lowestDistance];
            return returnValue;
        }
        public Shelf(int scaleFactor, Rectangle drawRectangle)
        {
            DrawRectangle = drawRectangle;
            SelectionRange = DrawRectangle;
            int additional = 1 * scaleFactor;
            SelectionRange.Width += additional;
            SelectionRange.Height += additional;
            SelectionRange.X -= additional / 2;
            SelectionRange.Y -= additional / 2;
        }
        public List<Vector2> GetPosistions()
        {
            if (ToiletPaperCount == 0)
                return new List<Vector2>();
            List<Vector2> returnValue = new List<Vector2>();
            int spacing = 5;
            int columns = DrawRectangle.Width / (spacing + TpSize.X);
            int currentColumn = 0;
            int initialX = DrawRectangle.X;
            int currentX = initialX;
            int currentY = DrawRectangle.Y;
            for (int i = 0; i < ToiletPaperCount; i++)
            {
                currentColumn++;
                returnValue.Add(
                    new Vector2(currentX, currentY)
                );
                currentX += spacing + TpSize.X;
                if (currentColumn == columns)
                {
                    currentColumn = 0;
                    currentY += TpSize.Y + spacing;
                    currentX = initialX;
                }
            }
            return returnValue;
        }
        public void Draw(SpriteBatch spriteBatch)
        {
            if (TpPosistions == null || TpPosistions.Count != ToiletPaperCount)
                TpPosistions = GetPosistions();
            spriteBatch.Draw(ShelfTexture, DrawRectangle, Color.White);
            foreach (Vector2 tpPosistion in TpPosistions)
                spriteBatch.Draw(TpTexture, new Rectangle(tpPosistion.ToPoint(), TpSize), Color.White);
            if (!Accessable) //TODO: Create some sort of method for calculating thickness
            {
                float thickness = (DrawRectangle.Width + DrawRectangle.Height) * 0.035f;
                spriteBatch.DrawRectangle(DrawRectangle, Color.LightBlue, thickness, 1);
            }
        }        
    }
}